#!/usr/bin/perl
# -*-perl-*-
#
# slashify.pl takes a file on the input stream and adds "\"
# marks to the end of consecutive lines after a #define.
# It backslash escapes newlines in your macros.
# 
# This is in the public domain, you are merely a jerk if you
# don't include this notice in a derived work. 
#

$tabLen        = 8;
$slashcolRound = 4;
$padSpaces     = 2;
$restoreTabs   = 1;

$debug = 1;

$inMacro = 0;

@macroBody = ();
$ln = 0; # line count 

while (<>) {
    chop;

    #print "*XXX* $_ *XXX*\n";

    if (/^\#\s*define/) {
	# If we come across a #define, it can either start
	# a new macro alone, or both end an old _and_ start a new.
	if ($inMacro) {
	    &printMacro(@macroBody);
	}
	# In both cases, start a new macro body
	@macroBody = (); # null array, add line later
	$inMacro = 1;
	$ln = 0;
	$macroBody[$ln++] = "$_"; # no trailing \n

    } elsif (/^\s*$/)  {
	# blank lines end previous define, print macro if we got one.
	&printMacro(@macroBody) if $inMacro;
	$inMacro = 0;
	# Print blank line out
	print "$_\n"; # add \n

    } elsif (/^\s*\#/) {
	# other pre-processor commands end the previous macro, if one
	&printMacro(@macroBody) if ($inMacro) ;
	$inMacro = 0;
	print "$_\n";

    } elsif ($inMacro) {
	# Record body of macro
	$macroBody[$ln++] = "$_"; # no trailing \n

    } else {
	# Other lines just get printed out directly
	print "$_\n";
    }
}

# If there's one last macro in progress, print it. EOF may have ended it.
&printMacro(@macroBody) if $inMacro;

sub printMacro {
    # The formal parameters.
    local(@lines) = @_;

    # Actual locals, not formal parameters
    local($i, $line, $ln, $slashcol);

    $slashcol = 0;

    ## Loop through all lines, find longest.  Also
    ## strip off trailing spaces and any trailing "\"
    for ($ln = 0; $ln <= $#lines; $ln++) {
	# Strip off trailing \, if any
	$lines[$ln] =~ s/\\\s*$//;
	# Strip off trailing space, if any
	$lines[$ln] =~ s/\s*$//;
	# Replace tabs with correct number of spaces
	$lines[$ln] =~ s/\t/' ' x ($tabLen - (length($`) % $tabLen))/ge;
	if (length($lines[$ln]) + $padSpaces> $slashcol) {
	    $slashcol = length($lines[$ln]) + $padSpaces;
	}
	#print "$slashcol::$line\n";
	#print "$lines[$ln]!\n"
    }

    ## Round slashCol up to nearest multiple of $slashcolRound
    $slashcol = 
	int(($slashcol + $slashcolRound) / $slashcolRound) * $slashcolRound;

    #print "/* Slash column ($slashcol): */\n";
    for ($ln = 0; $ln < $#lines; $ln++) {
	# compute number of spaces to insert at end before the \\
	$lineLen = length($lines[$ln]);
	$spaceCt = $slashcol - $lineLen;
	$spaces = ' ' x $spaceCt;

	#debugging:
	#print "$lines[$ln]($lineLen)\\$spaces\\($spaceCt)\n";
	
	# tack the spaces on the end
	$lines[$ln] = "$lines[$ln]$spaces";

	# Restore tabs, if requested
	#$lines[$ln] =~ s/( {$tabLen})/\t/og if $restoreTabs;

	# print the line
	print "$lines[$ln]\\\n";
    }
    # Last line is always unmodified.
    print "$lines[$#lines]\n";
}
